@@ -80,9 +80,36 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
 ### No need to have Ranges methods when mapping to transcripts. Plain
 ### ranges only make sense in the transcript space.
 
-.mapToTranscripts <- function(x, transcripts, hits, ignore.strand) 
+.mapToTranscripts <- function(x, transcripts, hits, 
+                              ignore.strand, intronJunctions=FALSE) 
 {
     flat <- unlist(transcripts, use.names=FALSE)
+    if (intronJunctions) {
+        ## introns must fall between ranges; not off the ends
+        nonHits <- !seq_along(x) %in% queryHits(hits)
+        if (any(nonHits)) {
+            follows <- follow(x[nonHits], flat, ignore.strand=ignore.strand)
+            precedes <- precede(x[nonHits], flat, ignore.strand=ignore.strand)
+            introns <- logical(length(nonHits))
+            introns[nonHits] <-  !is.na(follows & precedes)
+            if (any(introns)) {
+                ## modify 'x'
+                subjectIdx <- na.omit(follows[follows & precedes])
+                stopifnot(length(subjectIdx) == sum(introns))
+                nstrand <- as.logical(strand(flat[subjectIdx]) == "-")
+                ends <- end(flat[subjectIdx]) 
+                ends[nstrand] <- start(flat[subjectIdx[nstrand]]) - 1L
+                ranges(x[introns]) <- IRanges(end=ends, width=0L)
+                ## modify 'hits'
+                query <- c(queryHits(hits), seq_along(x)[introns])
+                subject <- c(subjectHits(hits), subjectIdx) 
+                intronic <- c(logical(length(queryHits(hits))), 
+                                !logical(sum(introns)))
+                hits <- Hits(query, subject, length(x), length(flat),
+                             intronic=intronic, sort.by.query=TRUE)
+            }
+        } else mcols(hits)$intronic <- logical(length(hits))
+    }
     if (length(hits)) {
         xHits <- queryHits(hits)
         txHits <- subjectHits(hits)
@@ -98,14 +125,20 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
             shifted <- .listCumsumShifted(width(transcripts))
             xrange <- shift(xrange, shifted[txHits])
         }
+
         ## seqnames come from 'transcripts'
-        txGroup <- togroup(PartitioningByWidth(transcripts))[txHits]
-        GRanges(names(transcripts)[txGroup], 
-                xrange, strand(flat)[txHits],
-                DataFrame(xHits, transcriptsHits=txGroup))
+        transcriptsHits=togroup(PartitioningByWidth(transcripts))[txHits]
+        df <- DataFrame(xHits=xHits, transcriptsHits=transcriptsHits)
+        if (intronJunctions)
+            df$intronic <- mcols(hits)$intronic
+        GRanges(names(transcripts)[transcriptsHits],
+                xrange, strand(flat)[txHits], df)
     } else {
         ans <- GRanges()
-        mcols(ans) <- DataFrame(xHits=integer(), transcriptsHits=integer())
+        df <- DataFrame(xHits=integer(), transcriptsHits=integer())
+        if (intronJunctions)
+            df$intronic <- logical()
+        mcols(ans) <- df 
         ans
     }
 }
@@ -120,7 +153,7 @@ setMethod("mapToTranscripts", c("GenomicRanges", "GenomicRanges"),
 )
 
 setMethod("mapToTranscripts", c("GenomicRanges", "GRangesList"), 
-    function(x, transcripts, ignore.strand=FALSE) 
+    function(x, transcripts, ignore.strand=FALSE, intronJunctions=FALSE) 
     {
         if (!length(x) && !length(transcripts))
             return(GRanges(xHits=integer(), transcriptsHits=integer()))
@@ -140,7 +173,7 @@ setMethod("mapToTranscripts", c("GenomicRanges", "GRangesList"),
         ## findOverlaps determines pairs
         hits <- findOverlaps(x, unlist(transcripts, use.names=FALSE), 
                              type="within", ignore.strand=ignore.strand)
-        .mapToTranscripts(x, transcripts, hits, ignore.strand)
+        .mapToTranscripts(x, transcripts, hits, ignore.strand, intronJunctions)
     }
 )
 
